{*********************************************************}
{                                                         }
{    HydroSoft Controls Library                           }
{    released into the public domain March 1998           }
{                                                         }
{*********************************************************}

unit HSUtils;

interface
   uses SysUtils, WinProcs, WinTypes, FileCtrl, Graphics, Classes;

type
  TDriveRange = 'A'..'Z';
  TDriveFlag = (dfValid, dfFloppy, dfFixed, dfNetwork, dfCDROM, dfRAM,
                dfRemoveable, dfWriteable);
  TDriveFlags = set of TDriveFlag;
  TFontDevice = (fdScreen, fdPrinter, fdBoth);
  TFontListOption = (flAnsiOnly, flTrueTypeOnly, flFixedPitchOnly,
  flNoOEMFonts, flOEMFontsOnly, flScalableOnly);
  TFontListOptions = set of TFontListOption;

const
  DriveDesc : array[TDriveType] of string[15] =
    ('Unknown drive', 'No drive', 'Floppy drive', 'Hard drive', 'Network drive',
     'CD-ROM drive', 'RAM disk');

var
  DriveFlags  : array[TDriveRange] of TDriveFlags;
  DriveTypes  : array[TDriveRange] of TDriveType;
  ValidDrives : set of TDriveRange;

function DriveNumber(Drive : Char) : Integer;
function WinDriveNumber(Drive: Char): Integer;
function FindDriveType(Drive: Char): TDriveType;
function GuessDriveType(Drive : Char) : TDriveType;
function GetDriveFlags(drive: Char) : TDriveFlags;
function IsRAMDrive(DriveNum: Integer): Boolean;
function MaxInt(const Values: array of Longint): Longint;
function GetItemHeight(Font: TFont): Integer;
function IsCDROM(DriveNum: Integer): Boolean;
function GetNetworkVolume(Drive: Char): string;
function GetVolumeID(Drive: Char): string;
procedure DetectCompyterDrives;

implementation

function DriveNumber(Drive : Char) : Integer;
begin
  Result := Ord(UpCase(Drive)) - Ord('A') + 1;
end;


function WinDriveNumber(Drive: Char): Integer;
begin
  Result := DriveNumber(Drive) - 1;
end;


function IsCDROM(DriveNum: Integer): Boolean; assembler;
asm
  MOV   AX, 1500h
  XOR   BX, BX
  INT   2Fh
  OR    BX, BX
  JZ    @Finish
  MOV   AX, 150Bh
  MOV   CX, DriveNum
  INT   2fh
  OR    AX, AX
  @Finish:
end;


function IsRAMDrive(DriveNum: Integer): Boolean; assembler;
var
  Temp: Boolean;
asm
  MOV   Temp, False
  PUSH  DS
  MOV   BX, SS
  MOV   DS, BX
  SUB   SP, 0200h
  MOV   BX, SP
  MOV   AX, DriveNum
  MOV   CX, 1
  XOR   DX, DX
  INT   25h
  ADD   SP, 2
  JC    @@1
  MOV   BX, SP
  CMP   BYTE PTR SS:[BX+15h], 0F8h
  JNE   @@1
  CMP   BYTE PTR SS:[BX+10h], 1 
  JNE   @@1
  MOV   Temp, True
@@1:
  ADD   SP, 0200h
  POP   DS
  MOV   AL, Temp
end;


function FindDriveType(Drive: Char): TDriveType;
var
  n : Integer;
begin
  n := WinDriveNumber(Drive);

  case GetDriveType(n) of
    0               : Result := dtNoDrive;
    DRIVE_REMOVABLE : Result := dtFloppy;
    DRIVE_FIXED     : if IsRAMDrive(n) then Result := dtRAM
                      else Result := dtFixed;
    DRIVE_REMOTE    : if IsCDROM(n) then Result := dtCDROM
                      else Result := dtNetwork;
  end;
end;



procedure DetectCompyterDrives;
var
  d: Char;
begin
  ValidDrives := [];
  for d := 'A' to 'Z' do begin
    DriveTypes[d] := FindDriveType(d);
    case DriveTypes[d] of
      dtUnknown : DriveFlags[d] := [dfValid, dfFixed, dfWriteable];
      dtNoDrive : DriveFlags[d] := [];
      dtFloppy  : DriveFlags[d] := [dfValid, dfFloppy, dfRemoveable, dfWriteable];
      dtFixed   : DriveFlags[d] := [dfValid, dfFixed, dfWriteable];
      dtNetwork : DriveFlags[d] := [dfValid, dfNetwork, dfWriteable];
      dtCDROM   : DriveFlags[d] := [dfValid, dfCDROM, dfRemoveable];
      dtRAM     : DriveFlags[d] := [dfValid, dfRAM, dfWriteable];
    end;
    if dfValid in DriveFlags[d] then Include(ValidDrives, d);
  end;
end;


function GuessDriveType(Drive : Char) : TDriveType;
begin
  Drive := UpCase(Drive);
  if Drive in ['A'..'Z'] then Result := DriveTypes[Drive]
  else Result := dtNoDrive;
end;


function GetDriveFlags(Drive: Char) : TDriveFlags;
begin
  Drive := UpCase(Drive);
  if Drive in ['A'..'Z'] then Result := DriveFlags[Drive]
  else Result := [];
end;


function GetVolumeID(Drive: Char): string;
var
  SR: TSearchRec;
begin
  Result := '';
  if FindFirst(Drive + ':\*.*', faVolumeID, SR) = 0 then
    Result := Uppercase(SR.Name);
end;


function GetNetworkVolume(Drive: Char): string;
const
  LocalName: array[0..2] of Char = 'C:'#0;
var
  BufSize: Word;
  Temp: array[0..128] of Char;
begin
  LocalName[0] := Drive;
  BufSize := 127;
  if WNetGetConnection(LocalName, Temp, @BufSize) = WN_SUCCESS then
    Result := StrPas(Temp)
  else
    Result := GetVolumeID(Drive);
end;

function MaxInt(const Values: array of Longint): Longint;
var
  I: Cardinal;
begin
  Result := Values[0];
  for I := 0 to High(Values) do
    if Values[I] > Result then Result := Values[I];
end;

function GetItemHeight(Font: TFont): Integer;
var
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  try
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
  finally
    ReleaseDC(0, DC);
  end;
  Result := Metrics.tmHeight + 3;
end;

end.
